@@ -19,7 +19,7 @@
 #include "smalloc.h"
 #include "filehash.h"
 
-static char fio_version_string[] = "fio 1.24";
+static char fio_version_string[] = "fio 1.25";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -206,6 +206,13 @@ static int setup_rate(struct thread_data *td)
 	return 0;
 }
 
+static int fixed_block_size(struct thread_options *o)
+{
+	return o->min_bs[DDIR_READ] == o->max_bs[DDIR_READ] &&
+		o->min_bs[DDIR_WRITE] == o->max_bs[DDIR_WRITE] &&
+		o->min_bs[DDIR_READ] == o->min_bs[DDIR_WRITE];
+}
+
 /*
  * Lazy way of fixing up options that depend on each other. We could also
  * define option callback handlers, but this is easier.
@@ -266,11 +273,28 @@ static int fixup_options(struct thread_data *td)
 
 	o->rw_min_bs = min(o->min_bs[DDIR_READ], o->min_bs[DDIR_WRITE]);
 
+	/*
+	 * For random IO, allow blockalign offset other than min_bs.
+	 */
+	if (!o->ba[DDIR_READ] || !td_random(td))
+		o->ba[DDIR_READ] = o->min_bs[DDIR_READ];
+	if (!o->ba[DDIR_WRITE] || !td_random(td))
+		o->ba[DDIR_WRITE] = o->min_bs[DDIR_WRITE];
+
+	if ((o->ba[DDIR_READ] != o->min_bs[DDIR_READ] ||
+	    o->ba[DDIR_WRITE] != o->min_bs[DDIR_WRITE]) &&
+	    !td->o.norandommap) {
+		log_err("fio: Any use of blockalign= turns off randommap\n");
+		td->o.norandommap = 1;
+	}
+
 	if (!o->file_size_high)
 		o->file_size_high = o->file_size_low;
 
-	if (o->norandommap && o->verify != VERIFY_NONE) {
-		log_err("fio: norandommap given, verify disabled\n");
+	if (o->norandommap && o->verify != VERIFY_NONE
+	    && !fixed_block_size(o))  {
+		log_err("fio: norandommap given for variable block sizes, "
+			"verify disabled\n");
 		o->verify = VERIFY_NONE;
 	}
 	if (o->bs_unaligned && (o->odirect || td->io_ops->flags & FIO_RAWIO))
@@ -384,13 +408,32 @@ static int exists_and_not_file(const char *filename)
 	return 1;
 }
 
+void td_fill_rand_seeds(struct thread_data *td)
+{
+	os_random_seed(td->rand_seeds[0], &td->bsrange_state);
+	os_random_seed(td->rand_seeds[1], &td->verify_state);
+	os_random_seed(td->rand_seeds[2], &td->rwmix_state);
+
+	if (td->o.file_service_type == FIO_FSERVICE_RANDOM)
+		os_random_seed(td->rand_seeds[3], &td->next_file_state);
+
+	os_random_seed(td->rand_seeds[5], &td->file_size_state);
+
+	if (!td_random(td))
+		return;
+
+	if (td->o.rand_repeatable)
+		td->rand_seeds[4] = FIO_RANDSEED * td->thread_number;
+
+	os_random_seed(td->rand_seeds[4], &td->random_state);
+}
+
 /*
  * Initialize the various random states we need (random io, block size ranges,
  * read/write mix, etc).
  */
 static int init_random_state(struct thread_data *td)
 {
-	unsigned long seeds[6];
 	int fd;
 
 	fd = open("/dev/urandom", O_RDONLY);
@@ -399,30 +442,15 @@ static int init_random_state(struct thread_data *td)
 		return 1;
 	}
 
-	if (read(fd, seeds, sizeof(seeds)) < (int) sizeof(seeds)) {
+	if (read(fd, td->rand_seeds, sizeof(td->rand_seeds)) <
+	    (int) sizeof(td->rand_seeds)) {
 		td_verror(td, EIO, "read");
 		close(fd);
 		return 1;
 	}
 
 	close(fd);
-
-	os_random_seed(seeds[0], &td->bsrange_state);
-	os_random_seed(seeds[1], &td->verify_state);
-	os_random_seed(seeds[2], &td->rwmix_state);
-
-	if (td->o.file_service_type == FIO_FSERVICE_RANDOM)
-		os_random_seed(seeds[3], &td->next_file_state);
-
-	os_random_seed(seeds[5], &td->file_size_state);
-
-	if (!td_random(td))
-		return 0;
-
-	if (td->o.rand_repeatable)
-		seeds[4] = FIO_RANDSEED * td->thread_number;
-
-	os_random_seed(seeds[4], &td->random_state);
+	td_fill_rand_seeds(td);
 	return 0;
 }
 
@@ -576,6 +604,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 		if (file_alloced) {
 			td_new->o.filename = NULL;
 			td_new->files_index = 0;
+			td_new->files_size = 0;
 			td_new->files = NULL;
 		}
 
@@ -598,7 +627,7 @@ static int skip_this_section(const char *name)
 	if (!strncmp(name, "global", 6))
 		return 0;
 
-	return strncmp(job_section, name, strlen(job_section));
+	return strcmp(job_section, name);
 }
 
 static int is_empty_or_comment(char *line)
