@@ -405,3 +405,11 @@ int td_io_close_file(struct thread_data *td, struct fio_file *f)
 
 	return put_file(td, f);
 }
+
+int td_io_get_file_size(struct thread_data *td, struct fio_file *f)
+{
+	if (!td->io_ops->get_file_size)
+		return 0;
+
+	return td->io_ops->get_file_size(td, f);
+}
