@@ -792,6 +792,16 @@ static struct fio_option options[] = {
 		.def	= "4k",
 		.parent = "rw",
 	},
+	{
+		.name	= "ba",
+		.alias	= "blockalign",
+		.type	= FIO_OPT_STR_VAL_INT,
+		.off1	= td_var_offset(ba[DDIR_READ]),
+		.off2	= td_var_offset(ba[DDIR_WRITE]),
+		.minval	= 1,
+		.help	= "IO block offset alignment",
+		.parent	= "rw",
+	},
 	{
 		.name	= "bsrange",
 		.alias	= "blocksize_range",
@@ -871,6 +881,10 @@ static struct fio_option options[] = {
 			    .oval = FIO_FSERVICE_RR,
 			    .help = "Round robin select files",
 			  },
+			  { .ival = "sequential",
+			    .oval = FIO_FSERVICE_SEQ,
+			    .help = "Finish one file before moving to the next",
+			  },
 		},
 		.parent = "nrfiles",
 	},
@@ -1285,6 +1299,13 @@ static struct fio_option options[] = {
 		.help	= "Fsync file after creation",
 		.def	= "1",
 	},
+	{
+		.name	= "create_on_open",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(create_on_open),
+		.help	= "Create files when they are opened for IO",
+		.def	= "0",
+	},
 	{
 		.name	= "cpuload",
 		.type	= FIO_OPT_INT,
